@@ -34,6 +34,10 @@ class InfiniteContext:
             from csv import reader, writer
             from statistics import median, stdev, variance
             from random import shuffle
+            from datetime import timedelta
+            from urllib.parse import urlparse
+            from os import makedirs
+            from os.path import join, exists
             self.__print_exc = print_exc
             self.__path, self.__environ, self.__walk, self.__remove = path, environ, walk, remove
             self.__gettempdir, self.__NamedTemporaryFile = gettempdir, NamedTemporaryFile
@@ -50,6 +54,10 @@ class InfiniteContext:
             self.__reader, self.__writer = reader, writer
             self.__median, self.__stdev, self.__variance = median, stdev, variance
             self.__shuffle = shuffle
+            self.__timedelta = timedelta
+            self.__urlparse = urlparse
+            self.__makedirs = makedirs
+            self.__join, self.__exists = join, exists
             try: from perpetual_context import PerpetualContext
             except:
                 installModule(module_name='perpetual-context', version='1.1.2')
@@ -88,8 +96,12 @@ class InfiniteContext:
                 installModule(module_name='opencv-python', version='4.6.0.66')
                 from cv2 import dnn, cvtColor, COLOR_RGB2BGR, imread, VideoCapture, CAP_PROP_FPS, CAP_PROP_FRAME_COUNT, CAP_PROP_POS_FRAMES, imwrite
             try:
-                from moviepy.editor import VideoFileClip
-                import moviepy.config as cfg
+                try:
+                    from moviepy.video.io.VideoFileClip import VideoFileClip
+                    import moviepy.config as cfg
+                except:
+                    from moviepy.editor import VideoFileClip
+                    import moviepy.config as cfg
             except:
                 installModule(module_name='moviepy', version='1.0.3')
                 from moviepy.editor import VideoFileClip
@@ -106,6 +118,10 @@ class InfiniteContext:
             except:
                 installModule(module_name='beautifulsoup4', version='4.12.3')
                 from bs4 import BeautifulSoup
+            try: from youtube_transcript_extractor import YoutubeTranscriptExtractor
+            except:
+                installModule(module_name='youtube-transcript-extractor', version='0.1.4')
+                from youtube_transcript_extractor import YoutubeTranscriptExtractor
             try: from fitz import open as fitz
             except:
                 installModule(module_name='PyMuPDF', version='1.24.5')
@@ -127,6 +143,7 @@ class InfiniteContext:
             self.__Recognizer, self.__AudioFile = Recognizer, AudioFile
             self.__AudioSegment = AudioSegment
             self.__BeautifulSoup = BeautifulSoup
+            self.__YoutubeTranscriptExtractor = YoutubeTranscriptExtractor
             self.__fitz = fitz
             self.__process = process
             self.__api_url = 'https://api.api-ninjas.com/v1/'
@@ -289,11 +306,35 @@ class InfiniteContext:
                 detected_objects_str = ', '.join(detected_objects) if len(detected_objects) > 0 else ''
                 return detected_objects_str.strip()
             def detectObjectsLocally(image_path):
+                def sapiens_download(download_path='./'):
+                    urls = [
+                        'https://huggingface.co/defaultmodels/default/resolve/main/sapiens.cfg?download=true',
+                        'https://huggingface.co/defaultmodels/default/resolve/main/sapiens.names?download=true',
+                        'https://huggingface.co/defaultmodels/default/resolve/main/sapiens.weights?download=true'
+                    ]
+                    if not self.__exists(download_path): self.__makedirs(download_path)
+                    try:
+                        self.__environ['SSL_CERT_FILE'] = self.__where()
+                        self.__getLogger('requests').setLevel(self.__ERROR)
+                    except: pass
+                    for address in urls:
+                        parsed = self.__urlparse(address)
+                        name = parsed.path.split('/')[-1]
+                        file_path = self.__join(download_path, name)
+                        stream = self.__urlopen(address)
+                        data = stream.read()
+                        file = open(file_path, 'wb')
+                        file.write(data)
+                        file.close()
                 def loadModel():
-                    net = self.__dnn.readNet('yolov3.weights', 'yolov3.cfg')
+                    root_directory = self.__getRootDirectory()
+                    if not self.__path.exists(root_directory+'sapiens.weights'): sapiens_download(download_path=root_directory)
+                    elif not self.__path.exists(root_directory+'sapiens.cfg'): sapiens_download(download_path=root_directory)
+                    elif not self.__path.exists(root_directory+'sapiens.names'): sapiens_download(download_path=root_directory)
+                    net = self.__dnn.readNet(root_directory+'sapiens.weights', root_directory+'sapiens.cfg')
                     layer_names = net.getLayerNames()
                     output_layers = [layer_names[i-1] for i in net.getUnconnectedOutLayers()]
-                    with open('coco.names', 'r') as file: classes = [line.strip() for line in file.readlines()]
+                    with open(root_directory+'sapiens.names', 'r') as file: classes = [line.strip() for line in file.readlines()]
                     return net, output_layers, classes
                 def loadImage(image_path):
                     if image_path.startswith('http://') or image_path.startswith('https://'):
@@ -363,6 +404,8 @@ class InfiniteContext:
                 return complete_text.strip()
             def detectTextsLocally(image_path):
                 self.__getLogger('ppocr').setLevel(self.__WARNING)
+                self.__getLogger('paddlex').setLevel(self.__WARNING)
+                self.__getLogger('paddle').setLevel(self.__WARNING)
                 optical_character_recognition = self.__PaddleOCR(use_angle_cls=True, lang=language)
                 if image_path.startswith('http://') or image_path.startswith('https://'):
                     try:
@@ -371,7 +414,8 @@ class InfiniteContext:
                     except: pass
                     response = self.__get(image_path)
                     image_path = self.__array(self.__Image.open(self.__BytesIO(response.content)))
-                results = optical_character_recognition.ocr(image_path, cls=True)
+                try: results = optical_character_recognition.ocr(image_path, cls=True)
+                except: results = optical_character_recognition.ocr(image_path)
                 try: extracted_text = ' '.join([linha[-1][0] for linha in results[0]])
                 except: extracted_text = ''
                 return extracted_text
@@ -711,7 +755,7 @@ class InfiniteContext:
                 publish_date_match = self.__search(r'"publishDate":"(.*?)"', response.text)
                 publish_date = publish_date_match.group(1) if publish_date_match else ''
                 return title.strip(), description.strip(), channel.strip(), publish_date.strip()
-            def getVideoTranscript(url):
+            def getVideoTranscriptLEGACY(url=''):
                 full_transcript = ''
                 response = self.__get(url)
                 if response.status_code == 200:
@@ -734,8 +778,26 @@ class InfiniteContext:
                                 full_transcript = ' '.join(transcript)
                                 full_transcript = '**In the video you can hear the following:** '+full_transcript
                 return full_transcript.strip()
+            def getVideoTranscript(url=''):
+                transcription = ''
+                def seconds_to_hhmmss(seconds=0.0):
+                    seconds = float(seconds)
+                    total_seconds = int(seconds)
+                    return str(self.__timedelta(seconds=total_seconds)).rjust(8, '0')
+                youtube_transcript_extractor = self.__YoutubeTranscriptExtractor(url)
+                clean_transcript = youtube_transcript_extractor.clean_transcript()
+                if type(clean_transcript) in (tuple, list) and len(clean_transcript) > 0:
+                    for extractor in clean_transcript:
+                        text = str(extractor.get('text', '')).strip()
+                        start = float(extractor.get('start', 0.0))
+                        duration = float(extractor.get('duration', 0.0))
+                        end = float(extractor.get('end', 0.0))
+                        transcription += f'**start:** {seconds_to_hhmmss(start)}, **end:** {seconds_to_hhmmss(end)}, **duration:** {seconds_to_hhmmss(duration)}\n**speech:** {text}\n\n'
+                transcription = transcription.strip()
+                return transcription
             result_text, text_complements = getVideoTranscript(url=file_path), ''
-            if len(result_text) < 1: return self.__perpetual_context.getSummaryYouTube(file_path=file_path, max_tokens=max_tokens)
+            if len(result_text.strip()) < 1: result_text = self.__perpetual_context.getSummaryYouTube(file_path=file_path, max_tokens=max_tokens)
+            if len(result_text.strip()) < 1: result_text = getVideoTranscriptLEGACY(file_path=file_path)
             title, description, channel, publish_date = getVideoInfo(url=file_path)
             if len(channel) > 0: text_complements += f'Name of this YouTube channel: **{channel}**\n\n'
             if len(title) > 0: text_complements += f'## {title}\n\n'
